#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <stdio.h>

/* TI Driver Header Files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/timer/GPTimerCC26XX.h>
#include <ti/devices/cc13x0/driverlib/ioc.h>
#include <ti/sysbios/knl/Task.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/knl/Clock.h>
#include "ti_drivers_config.h"

/* Board Header files */
#include "Board.h"

/* Global Variables */
GPTimerCC26XX_Handle hTimer;
GPTimerCC26XX_Params timerParams;

volatile uint32_t pulseTimePrev = 0;
volatile uint32_t pulseTimeCurr = 0;

/* Timer Capture Interrupt Callback */
void timerCallback(GPTimerCC26XX_Handle handle, GPTimerCC26XX_IntMask interruptMask) {
    pulseTimePrev = pulseTimeCurr;
    pulseTimeCurr = GPTimerCC26XX_getValue(handle);
}

/* Initialize GPTimer for Pulse Capture */
void initPulseCapture() {
    GPTimerCC26XX_Params_init(&timerParams);
    timerParams.mode = GPT_MODE_EDGE_COUNT;  // Capture Mode
    timerParams.width = GPT_CONFIG_16BIT;
    timerParams.direction = GPTimerCC26XX_DIRECTION_UP;
    timerParams.debugStallMode = GPTimerCC26XX_DEBUG_STALL_OFF;

    hTimer = GPTimerCC26XX_open(CONFIG_GPTIMER_0, &timerParams);
    if (hTimer == NULL) {
        System_printf("❌ Error: GPTimer Open Failed\n");
        System_flush();
        return;
    }

    // Map IOID_13 to GPTimer
    GPTimerCC26XX_PinMux pinMux = GPTimerCC26XX_getPinMux(hTimer);
    IOCPortConfigureSet(IOID_13, pinMux, IOC_STD_INPUT);
    System_printf("✅ IOID_13 Mapped to GPTimer\n");
    System_flush();

    // Register Timer Interrupt
    GPTimerCC26XX_registerInterrupt(hTimer, timerCallback, GPT_INT_CAPTURE);
    System_printf("✅ Timer Interrupt Registered\n");
    System_flush();

    // Capture on Both Edges (For More Accurate Counting)
    GPTimerCC26XX_setCaptureEdge(hTimer, GPTimerCC26XX_BOTH_EDGES);

    // Start Timer
    GPTimerCC26XX_start(hTimer);
    System_printf("✅ GPTimer Started!\n");
    System_flush();
}

/* Pulse Counting Routine */
void pulseCountingRoutine() {
    uint32_t pulseCount = 0;  // Reset pulse count
    GPIO_write(Board_GPIO_BTN2, 1); // Set BTN2 high
    // Wait for 50ms before starting count
    Task_sleep(50 * 1000 / Clock_tickPeriod);

    // Start Counting for 60ms
    uint32_t startValue = GPTimerCC26XX_getValue(hTimer);
    Task_sleep(17 * 1000 / Clock_tickPeriod);
    uint32_t endValue = GPTimerCC26XX_getValue(hTimer);

    // Calculate pulses counted
    pulseCount = endValue - startValue;
    GPIO_write(Board_GPIO_BTN2, 0); // Set BTN2 Low
    // Print the final pulse count
    System_printf("🔄 Pulse Count in 60ms: %u pulses\n", pulseCount);
    System_flush();
}

/* Main Thread */
void *mainThread(void *arg0) {
    GPIO_init();
    initPulseCapture();  // Initialize GPTimer
    // Initialize GPIO for BTN2
    GPIO_setConfig(Board_GPIO_BTN2, GPIO_CFG_OUTPUT | GPIO_CFG_OUT_HIGH); // Configure as output, start low
    GPIO_write(Board_GPIO_BTN2, 0); // Set BTN2 Low

    while (1) {
        pulseCountingRoutine();  // Run Pulse Counting Routine
    }
}
